# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from dateutil.relativedelta import relativedelta

from odoo import _, api, fields, models
from odoo.exceptions import UserError


class EticPartner(models.Model):
    _inherit = "res.partner"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    is_shop = fields.Boolean(
        string="Est un commerce",
        compute="_compute_is_shop",
        store=True,
    )
    all_stock_move_ids = fields.Many2many(
        comodel_name="stock.move",
        string="Tous les Transferts",
        compute="_compute_all_stock_move_ids",
    )
    monthly_stock_move_ids = fields.Many2many(
        comodel_name="stock.move",
        string="Transferts de ce mois",
        compute="_compute_monthly_stock_move_ids",
    )
    stock_move_count = fields.Integer(
        string="Nombre de transferts", compute="_compute_stock_move_count"
    )

    currency_id = fields.Many2one(
        comodel_name="res.currency",
        default=lambda self: self.env.company.currency_id.id,
    )

    consigne_balance = fields.Float(
        string="Solde", compute="_compute_consigne_balance", digits=0
    )
    amount_caisses = fields.Integer(
        string="Caisses chez commerce", compute="_compute_amount_caisses"
    )
    monthly_delivered_caisse = fields.Integer(
        string="Caisses livrées ce mois", compute="_compute_monthly_delivered_caisse"
    )

    # ------------------------------------------------------
    # Compute
    # ------------------------------------------------------
    @api.depends("type_id")
    def _compute_is_shop(self):
        for r in self:
            r.is_shop = r.type_id == self.env.ref("etic_partner.type_shop")

    def _compute_all_stock_move_ids(self):
        for r in self:
            r.all_stock_move_ids = self.env["stock.move"].search(
                [
                    ("picking_partner_id", "=", r.id),
                ],
                order="product_id, date desc",
            )

    @api.depends("all_stock_move_ids")
    def _compute_monthly_stock_move_ids(self):
        month_start = fields.Datetime.today() + relativedelta(day=1)
        month_end = fields.Datetime.today() + relativedelta(months=1, day=1, days=-1)
        for r in self:
            r.monthly_stock_move_ids = r.all_stock_move_ids.filtered(
                lambda d: d.date >= month_start and d.date <= month_end
            )

    @api.depends("all_stock_move_ids")
    def _compute_stock_move_count(self):
        for r in self:
            r.stock_move_count = len(r.all_stock_move_ids)

    @api.depends("all_stock_move_ids")
    def _compute_consigne_balance(self):
        for r in self:
            r.consigne_balance = r.deposit_balance + sum(
                r.all_stock_move_ids.mapped("amount_consigne_relative")
            )

    @api.depends("all_stock_move_ids")
    def _compute_amount_caisses(self):
        cat_product = self.env.company.product_caisse_etic_id
        for r in self:
            stock_moves = sum(
                r.all_stock_move_ids.filtered(
                    lambda d: d.product_id.categ_id.id == cat_product.id
                    and d.state == "done"
                ).mapped("product_uom_relative_qty_done")
            )
            r.amount_caisses = r.intial_boxes + stock_moves

    @api.depends("monthly_stock_move_ids")
    def _compute_monthly_delivered_caisse(self):
        cat_product = self.env.company.product_caisse_etic_id
        for r in self:
            r.monthly_delivered_caisse = sum(
                r.monthly_stock_move_ids.filtered(
                    lambda d: d.picking_code == "outgoing"
                    and d.product_id.categ_id.id == cat_product.id
                    and d.state == "done"
                ).mapped("product_uom_relative_qty_done")
            )

    # ------------------------------------------------------
    # Buttons
    # ------------------------------------------------------
    def button_suivi_commerce(self):
        """
        Ouvre la vue suivie commerce
        """
        action = {
            "type": "ir.actions.act_window",
            "name": "Suivi Commerce",
            "views": [
                [
                    self.env.ref(
                        "etic_partner_stock.etic_res_partner_stock_form_view"
                    ).id,
                    "form",
                ]
            ],
            "view_mode": "form",
            "res_model": "res.partner",
            "res_id": self.id,
            "target": "current",
        }
        return action

    def button_form_contact(self):
        """
        Ouvre la vue contact
        """
        action = {
            "type": "ir.actions.act_window",
            "name": "Suivi Commerce",
            "views": [[False, "form"]],
            "view_mode": "form",
            "res_model": "res.partner",
            "res_id": self.id,
            "target": "current",
        }
        return action

    def button_details_transferts(self):
        """
        Ouvre la vue détail des transferts (stock.move)
        """
        action = {
            "type": "ir.actions.act_window",
            "name": "Détails transferts",
            "views": [
                [self.env.ref("etic_partner_stock.etic_stock_move_tree").id, "tree"]
            ],
            "search_view_id": [
                self.env.ref("etic_partner_stock.etic_stock_move_search").id
            ],
            "view_mode": "tree",
            "res_model": "stock.move",
            "target": "current",
            "context": {
                "search_default_picking_partner_id": self.id,
            },
        }
        return action

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def create_delivery_picking(self):
        """
        - Check si l'opération de picking est bien configurée
        """
        if not self.env.company.picking_type_delivery_etic_id:
            raise UserError(_("L'opération \"Livraison\" n'a pas été " "configurée !"))
        else:
            picking_type_id = self.env.company.picking_type_delivery_etic_id
            return self.create_picking(picking_type_id)

    def create_return_picking(self):
        """
        - Check si l'opération de picking est bien configurée
        """
        if not self.env.company.picking_type_return_etic_id:
            raise UserError(_("L'opération \"Livraison\" n'a pas été " "configurée !"))
        else:
            picking_type_id = self.env.company.picking_type_return_etic_id
            return self.create_picking(picking_type_id)

    def create_picking(self, type_picking):
        """
        - Création du stock.picking et ouverture de l'action
        """
        picking_id = self.env["stock.picking"].create(
            {
                "picking_type_id": type_picking.id,
                "location_id": type_picking.default_location_src_id.id,
                "location_dest_id": type_picking.default_location_dest_id.id,
                "partner_id": self.id,
            }
        )
        action = {
            "type": "ir.actions.act_window",
            "name": "Opération",
            "views": [[False, "form"]],
            "view_mode": "form",
            "res_model": "stock.picking",
            "target": "current",
            "res_id": picking_id.id,
            "flags": {"mode": "edit"},
        }
        return action
